@@ -176,8 +176,10 @@ MCRYPT mcrypt_module_open(char *algorithm,
 	ret = mcrypt_dlopen( &td->algorithm_handle, a_directory, m_directory, algorithm);
 	if (ret == NULL) {
 		err=lt_dlerror();
-		if (err!=NULL) fputs( err, stderr);
-		fputs("\n", stderr);
+		if (err!=NULL) {
+			fputs( err, stderr);
+			fputs("\n", stderr);
+		}
 		free(td);
 		return MCRYPT_FAILED;
 	}
@@ -186,7 +188,10 @@ MCRYPT mcrypt_module_open(char *algorithm,
 
 	if (ret == NULL) {
 		err=lt_dlerror();
-		if (err!=NULL) fputs( err, stderr);
+		if (err!=NULL) {
+			fputs( err, stderr);
+			fputs( "\n", stderr);
+		}
 		mcrypt_dlclose(td->algorithm_handle);
 		free(td);
 		return MCRYPT_FAILED;
@@ -199,9 +204,22 @@ MCRYPT mcrypt_module_open(char *algorithm,
 	td->a_block_size =
 	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_block_size");
 
+	if (td->a_encrypt == NULL || td->a_decrypt == NULL|| td->m_encrypt == NULL||
+		td->m_decrypt == NULL|| td->a_block_size == NULL) {
+		err=lt_dlerror();
+		if (err!=NULL) {
+			fputs( err, stderr);
+			fputs( "\n", stderr);
+		}
+		mcrypt_dlclose(td->algorithm_handle);
+		free(td);
+		return MCRYPT_FAILED;
+	}
+
 	if (mcrypt_enc_is_block_algorithm_mode(td) !=
 	    mcrypt_enc_is_block_algorithm(td)) {
 		mcrypt_module_close(td);
+		free(td);
 		return MCRYPT_FAILED;
 	}
 
@@ -222,8 +240,10 @@ int mcrypt_get_size(MCRYPT td)
 	_mcrypt_get_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_size");
 	if (_mcrypt_get_size == NULL) {
 		error = lt_dlerror();
-		if (error!=NULL) fputs(error, stderr);
-		fputs("\n", stderr);
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 	return _mcrypt_get_size();
@@ -239,8 +259,10 @@ int mcrypt_mode_get_size(MCRYPT td)
 	_mcrypt_get_size = mcrypt_dlsym(td->mode_handle, "_mcrypt_mode_get_size");
 	if (_mcrypt_get_size == NULL) {
 		error = lt_dlerror();
-		if (error!=NULL) fputs(error, stderr);
-		fputs("\n", stderr);
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 	return _mcrypt_get_size();
@@ -258,8 +280,10 @@ int mcrypt_set_key(MCRYPT td, void *a, void *b, int c, void *d, int e)
 		__mcrypt_set_key_stream = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
 		if (__mcrypt_set_key_stream == NULL) {
 			error = lt_dlerror();
-			if (error!=NULL) fputs(error, stderr);
-			fputs("\n", stderr);
+			if (error!=NULL) {
+				fputs(error, stderr);
+				fputs("\n", stderr);
+			}
 			return -2;
 		}
 		return __mcrypt_set_key_stream(a, b, c, d, e);
@@ -267,8 +291,10 @@ int mcrypt_set_key(MCRYPT td, void *a, void *b, int c, void *d, int e)
 		__mcrypt_set_key_block = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_set_key");
 		if (__mcrypt_set_key_block == NULL) {
 		 	error = lt_dlerror();
-			if (error!=NULL) fputs(error, stderr);
-			fputs("\n", stderr);
+			if (error!=NULL) {
+				fputs(error, stderr);
+				fputs("\n", stderr);
+			}
 			return -2;
 		}
 		return __mcrypt_set_key_block(a, b, c);
@@ -284,8 +310,10 @@ int mcrypt_enc_set_state(MCRYPT td, void *iv, int size)
 	__mcrypt_set_state = mcrypt_dlsym(td->mode_handle, "_mcrypt_set_state");
 	if (__mcrypt_set_state==NULL) {
 		error = lt_dlerror();
-		if (error!=NULL) fputs(error, stderr);
-		fputs("\n", stderr);
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 	return __mcrypt_set_state(td->abuf, iv, size);
@@ -303,9 +331,18 @@ int mcrypt_enc_get_block_size(MCRYPT td)
 WIN32DLL_DEFINE
 int mcrypt_get_algo_iv_size(MCRYPT td)
 {
+	const char* error;
 	int (*_mcrypt_get_algo_iv_size) (void);
 
 	_mcrypt_get_algo_iv_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_algo_iv_size");
+	if (_mcrypt_get_algo_iv_size == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 	return _mcrypt_get_algo_iv_size();
 }
 
@@ -322,9 +359,18 @@ int mcrypt_enc_get_iv_size(MCRYPT td)
 WIN32DLL_DEFINE
 int mcrypt_enc_get_key_size(MCRYPT td)
 {
+	const char* error;
 	int (*_mcrypt_get_key_size) (void);
 
 	_mcrypt_get_key_size = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_key_size");
+	if (_mcrypt_get_key_size == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 	return _mcrypt_get_key_size();
 }
 
@@ -333,9 +379,20 @@ int *mcrypt_enc_get_supported_key_sizes(MCRYPT td, int *len)
 {
 	int *(*_mcrypt_get_key_sizes) (int *);
 	int *size, *ret;
+	const char* error;
 
 	_mcrypt_get_key_sizes =
 	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_supported_key_sizes");
+	if (_mcrypt_get_key_sizes == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		*len = 0;
+		return NULL;
+	}
+
 	size = _mcrypt_get_key_sizes(len);
 	
 	ret = NULL;
@@ -351,8 +408,18 @@ WIN32DLL_DEFINE
 int mcrypt_enc_is_block_algorithm(MCRYPT td)
 {
 	int (*_is_block_algorithm) (void);
+	const char* error;
 
 	_is_block_algorithm = mcrypt_dlsym(td->algorithm_handle, "_is_block_algorithm");
+	if (_is_block_algorithm == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _is_block_algorithm();
 }
 
@@ -360,9 +427,19 @@ WIN32DLL_DEFINE
 char *mcrypt_enc_get_algorithms_name(MCRYPT td)
 {
 	const char *(*_mcrypt_get_algorithms_name) (void);
+	const char* error;
 
 	_mcrypt_get_algorithms_name =
 	    mcrypt_dlsym(td->algorithm_handle, "_mcrypt_get_algorithms_name");
+	if (_mcrypt_get_algorithms_name == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return NULL;
+	}
+
 	return strdup(_mcrypt_get_algorithms_name());
 }
 
@@ -370,8 +447,18 @@ WIN32DLL_DEFINE
 int init_mcrypt(MCRYPT td, void *buf, void *a, int b, void *c)
 {
 	int (*_init_mcrypt) (void *, void *, int, void *, int);
+	const char* error;
 
 	_init_mcrypt = mcrypt_dlsym(td->mode_handle, "_init_mcrypt");
+	if (_init_mcrypt == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _init_mcrypt(buf, a, b, c, mcrypt_enc_get_block_size(td));
 }
 
@@ -379,8 +466,18 @@ WIN32DLL_DEFINE
 int end_mcrypt(MCRYPT td, void *buf)
 {
 	int (*_end_mcrypt) (void *);
+	const char* error;
 
 	_end_mcrypt = mcrypt_dlsym(td->mode_handle, "_end_mcrypt");
+	if (_end_mcrypt == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _end_mcrypt(buf);
 }
 
@@ -408,8 +505,18 @@ WIN32DLL_DEFINE
 char *mcrypt_enc_get_modes_name(MCRYPT td)
 {
 	const char *(*_mcrypt_get_modes_name) (void);
+	const char* error;
 
 	_mcrypt_get_modes_name = mcrypt_dlsym(td->mode_handle, "_mcrypt_get_modes_name");
+	if (_mcrypt_get_modes_name == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return NULL;
+	}
+
 	return strdup(_mcrypt_get_modes_name());
 }
 
@@ -417,8 +524,18 @@ WIN32DLL_DEFINE
 int mcrypt_enc_is_block_mode(MCRYPT td)
 {
 	int (*_is_block_mode) (void);
-
+	const char* error;
+	
 	_is_block_mode = mcrypt_dlsym(td->mode_handle, "_is_block_mode");
+	if (_is_block_mode == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _is_block_mode();
 }
 
@@ -426,8 +543,18 @@ WIN32DLL_DEFINE
 int mcrypt_enc_mode_has_iv(MCRYPT td)
 {
 	int (*_has_iv) (void);
+	const char* error;
 
 	_has_iv = mcrypt_dlsym(td->mode_handle, "_has_iv");
+	if (_has_iv == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _has_iv();
 }
 
@@ -435,8 +562,18 @@ WIN32DLL_DEFINE
 int mcrypt_enc_is_block_algorithm_mode(MCRYPT td)
 {
 	int (*_is_a_block_mode) (void);
+	const char* error;
 
 	_is_a_block_mode = mcrypt_dlsym(td->mode_handle, "_is_block_algorithm_mode");
+	if (_is_a_block_mode == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+
 	return _is_a_block_mode();
 }
 
@@ -444,8 +581,17 @@ WIN32DLL_DEFINE
 int mcrypt_enc_self_test(MCRYPT td)
 {
 	int (*_self_test) (void);
+	const char* error;
 
 	_self_test = mcrypt_dlsym(td->algorithm_handle, "_mcrypt_self_test");
+	if (_self_test == NULL) {
+		error = lt_dlerror();
+		if (error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	return _self_test();
 }
@@ -467,14 +613,21 @@ int mcrypt_module_self_test(char *algorithm, char *a_directory)
 
 	if (!rr) {
 		error=lt_dlerror();
-		if (error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if (error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_self_test = mcrypt_dlsym(_handle, "_mcrypt_self_test");
-
+	if (_self_test == NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
+	
 	i = _self_test();
 
 	mcrypt_dlclose(_handle);
@@ -501,12 +654,20 @@ int mcrypt_module_algorithm_version(char *algorithm, char *a_directory)
 	rr = mcrypt_dlopen(&_handle, a_directory, NULL, algorithm);
 	if (!rr) {
 		error=lt_dlerror();
-		if (error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if (error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
+		lt_dlexit();
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_version = mcrypt_dlsym(_handle, "_mcrypt_algorithm_version");
+	if (_version==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _version();
 
@@ -534,12 +695,19 @@ int mcrypt_module_mode_version(char *mode, char *m_directory)
 	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs(error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs(error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_version = mcrypt_dlsym( _handle, "_mcrypt_mode_version");
+	if (_version==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _version();
 
@@ -567,12 +735,19 @@ int mcrypt_module_is_block_algorithm(char *algorithm, char *a_directory)
 	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_is_block_algorithm = mcrypt_dlsym(_handle, "_is_block_algorithm");
+	if (_is_block_algorithm==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _is_block_algorithm();
 
@@ -600,12 +775,19 @@ int mcrypt_module_is_block_algorithm_mode(char *mode, char *m_directory)
 	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_is_a_block_mode = mcrypt_dlsym(_handle, "_is_block_algorithm_mode");
+	if (_is_a_block_mode==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _is_a_block_mode();
 
@@ -633,12 +815,19 @@ int mcrypt_module_is_block_mode(char *mode, char *m_directory)
 	rr = mcrypt_dlopen( &_handle, m_directory, NULL, mode);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_is_block_mode = mcrypt_dlsym(_handle, "_is_block_mode");
+	if (_is_block_mode==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _is_block_mode();
 
@@ -666,12 +855,19 @@ int mcrypt_module_get_algo_block_size(char *algorithm, char *a_directory)
 	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_get_block_size = mcrypt_dlsym(_handle, "_mcrypt_get_block_size");
+	if (_get_block_size==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _get_block_size();
 
@@ -699,12 +895,19 @@ int mcrypt_module_get_algo_key_size(char *algorithm, char *a_directory)
 	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		return MCRYPT_UNKNOWN_ERROR;
 	}
 
 	_get_key_size = mcrypt_dlsym(_handle, "_mcrypt_get_key_size");
+	if (_get_key_size==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return MCRYPT_UNKNOWN_ERROR;
+	}
 
 	i = _get_key_size();
 
@@ -722,7 +925,7 @@ int *mcrypt_module_get_algo_supported_key_sizes(char *algorithm,
 						int *len)
 {
 	mcrypt_dlhandle _handle;
-	const int *(*_mcrypt_get_key_sizes) (int *);
+	int *(*_mcrypt_get_key_sizes) (int *);
 	int *size;
 	int * ret_size;
 	const char* error;
@@ -736,14 +939,21 @@ int *mcrypt_module_get_algo_supported_key_sizes(char *algorithm,
 	rr = mcrypt_dlopen( &_handle, a_directory, NULL, algorithm);
 	if (!rr) {
 		error=lt_dlerror();
-		if(error!=NULL) fputs( error, stderr);
-		fputs("\n", stderr);
+		if(error!=NULL) {
+			fputs( error, stderr);
+			fputs("\n", stderr);
+		}
 		*len = 0;
 		return NULL;
 	}
 
 	_mcrypt_get_key_sizes =
 	    mcrypt_dlsym(_handle, "_mcrypt_get_supported_key_sizes");
+	if (_mcrypt_get_key_sizes==NULL) {
+		mcrypt_dlclose(_handle);
+		lt_dlexit();
+		return NULL;
+	}
 
 	ret_size = NULL;
 	size = _mcrypt_get_key_sizes(len);
